@@ -24,21 +24,20 @@
 
 "bgp" <-
 function(X, Z, XX=NULL, bprior="bflat", corr="expsep",
-         BTE=c(1000,4000,2), R=1, m0r1=FALSE,
-         pred.n=TRUE, ds2x=FALSE, ego=FALSE, nu=1.5,
-         traces=FALSE, verb=1)#, 
-# base="gp")
+         BTE=c(1000,4000,2), R=1, m0r1=FALSE, itemps=NULL,
+         pred.n=TRUE, Ds2x=FALSE, improv=FALSE, nu=1.5,
+         trace=FALSE, verb=1, ...)
+	 ##base="gp", ... )
 {
   n <- dim(X)[1]
   if(is.null(n)) { n <- length(X); X <- matrix(X, nrow=n); d <- 1 }
   else { d <- dim(X)[2] }
-  params <- tgp.default.params(d+1)#, base=base)
+  params <- tgp.default.params(d+1, ...)#base=base, ...)
   params$bprior <- bprior
   params$corr <- corr
-  params$tree <- c(0,0,10)	# no tree
+  params$tree <- c(0,0,params$tree[3])	# no tree
   params$gamma <- c(0,0.2,0.7)	# no llm
-
   if(corr == "matern") params$nu <- nu
-  return(tgp(X,Z,XX,BTE,R,m0r1,FALSE,params,pred.n,ds2x,ego,traces,verb))
+  return(tgp(X,Z,XX,BTE,R,m0r1,FALSE,params,itemps,pred.n,Ds2x,improv,trace,verb))
 }
 
