@@ -24,15 +24,15 @@
 
 "blm" <-
 function(X, Z, XX=NULL, bprior="bflat", BTE=c(1000,4000,3), R=1, m0r1=FALSE,
-         pred.n=TRUE, ds2x=FALSE, ego=FALSE, traces=FALSE, verb=1)
+         itemps=NULL,pred.n=TRUE, Ds2x=FALSE, improv=FALSE, trace=FALSE, verb=1, ...)
 {
   n <- dim(X)[1]
   if(is.null(n)) { n <- length(X); X <- matrix(X, nrow=n); d <- 1 }
   else { d <- dim(X)[2] }
-  params <- tgp.default.params(d+1)
+  params <- tgp.default.params(d+1, ...)
   params$bprior <- bprior
-  params$tree <- c(0,0,10)	# no tree
+  params$tree <- c(0,0,params$tree[3])	# no tree
   params$gamma <- c(-1,0.2,0.7)	# force llm
-  return(tgp(X,Z,XX,BTE,R,m0r1,FALSE,params,pred.n,ds2x,ego,traces,verb))
+  return(tgp(X,Z,XX,BTE,R,m0r1,FALSE,params,itemps,pred.n,Ds2x,improv,trace,verb))
 }
 
