@@ -70,26 +70,32 @@ class Gp : public Base
   virtual void Clear(void);
   virtual void ClearPred(void);
   virtual void Update(double **X, unsigned int n, unsigned int d, double *Z);
-  virtual void UpdatePred(double **XX, unsigned int nn, unsigned int d, 
-			  double **Ds2xy);  
+  virtual void UpdatePred(double **XX, unsigned int nn, unsigned int d, bool Ds2xy);  
   virtual bool Draw(void *state);
-  virtual void Predict(unsigned int n, unsigned int nn, double *z, double *zz, 
-		       double **ds2xy, double *ego, bool err, void *state);
+  virtual void Predict(unsigned int n, double *zp, double *zpm, double *zps2, 
+		       unsigned int nn, double *zz, double *zzm, double *zzs2,
+		       double **ds2xy, double *improv, double Zmin, bool err,
+		       void *state);
   virtual void Match(Base* gp_old);
-  virtual void Combine(Base *l_gp, Base *r_gp, void *state);
+  virtual void Combine(Base *l_gp, Base *r_gp,void *state);
   virtual void Split(Base *l_gp, Base *r_gp, void *state);
   virtual double Posterior(void);
+  virtual double MarginalLikelihood(double itemp);
+  virtual double Likelihood(double itemp);
+  virtual double FullPosterior(double itemp);
+  virtual double MarginalPosterior(double itemp);
   virtual void Compute(void);
   virtual void ToggleLinear(void);
   virtual bool Linear(void);
   virtual void printFullNode(void);
   virtual double Var(void);
-  virtual double FullPosterior(void);
   virtual char* State(void);
   virtual unsigned int sum_b(void);
-  virtual void Init(void);
+  virtual void Init(double *dgp);
   virtual void X_to_F(unsigned int n, double **X, double **F);
-  virtual double* Trace(unsigned int* len);
+  virtual double* Trace(unsigned int* len, bool full);
+  virtual char** TraceNames(unsigned int* len, bool full);
+  virtual double NewInvTemp(double itemp, bool isleaf);
 
   double* get_b(void);
   double *Bmle(void);
@@ -109,9 +115,9 @@ class Gp_Prior : public Base_Prior
   Corr_Prior *corr_prior;
   unsigned int col;		
 
-  double *b;		        /* col, regression coefficients */
-  double s2;		        /* variance parameter */
-  double tau2;		        /* linear variance parameter */
+  double *b;		        /* starting: col, GP linear regression coefficients */
+  double s2;		        /* starting: GP variance parameter */
+  double tau2;		        /* starting: GP linear variance parameter */
 
   double *b0;		        /* hierarchical non-tree parameter b0 */
   
@@ -149,6 +155,7 @@ class Gp_Prior : public Base_Prior
 
   virtual void read_ctrlfile(std::ifstream* ctrlfile);
   virtual void read_double(double *dparams);
+  virtual void Init(double *dhier);
   
   virtual void Draw(Tree** leaves, unsigned int numLeaves, void *state);
   virtual bool LLM(void);
@@ -158,6 +165,9 @@ class Gp_Prior : public Base_Prior
   virtual Base* newBase(Model *model);
   virtual Base_Prior* Dup(void);
   virtual double log_HierPrior(void);
+  virtual double* Trace(unsigned int* len, bool full);
+  virtual char** TraceNames(unsigned int* len, bool full);
+  virtual double GamLin(unsigned int which);
 
   void InitT(void);
   void read_beta(char *line);
@@ -181,9 +191,10 @@ class Gp_Prior : public Base_Prior
   BETA_PRIOR BetaPrior(void);
 };
 
-void allocate_leaf_params(unsigned int col, double ***b, double **s2, 
-			  double **tau2, Corr ***corr, Tree **leaves, 
+void allocate_leaf_params(unsigned int col, double ***b, double **s2, double **tau2, 
+			  unsigned int **n, Corr ***corr, Tree **leaves, 
 			  unsigned int numLeaves);
-void deallocate_leaf_params(double **b, double *s2, double *tau2, Corr **corr);
+void deallocate_leaf_params(double **b, double *s2, double *tau2, unsigned int *n,
+			    Corr **corr);
 
 #endif
