@@ -55,8 +55,8 @@ class Matern : public Corr
   virtual void Update(unsigned int n1, unsigned int n2, double **K, double **X, double **XX);
   virtual void Update(unsigned int n1, double **X);
   virtual void Update(unsigned int n1, double **K, double **X);
-  virtual int Draw(unsigned int n, double **F, double **X, double *Z, 
-		   double *lambda, double **bmu, double **Vb, double tau2, void *state);
+  virtual int Draw(unsigned int n, double **F, double **X, double *Z, double *lambda, 
+		   double **bmu, double **Vb, double tau2, double itemp, void *state);
   virtual void Combine(Corr *c1, Corr *c2, void *state);
   virtual void Split(Corr *c1, Corr *c2, void *state);
   virtual char* State(void);
@@ -64,9 +64,11 @@ class Matern : public Corr
   virtual unsigned int sum_b(void);
   virtual void ToggleLinear(void);
   virtual bool DrawNug(unsigned int n, double **X, double **F, double *Z,
-		       double *lambda, double **bmu, 
-		       double **Vb, double tau2, void *state);
+		       double *lambda, double **bmu, double **Vb, double tau2, 
+		       double itemp, void *state);
   virtual double* Trace(unsigned int* len);
+  virtual char** TraceNames(unsigned int* len);
+  virtual void Init(double *dmat);
 
   void get_delta_d(Matern* c1, Matern* c2, void *state);
   void propose_new_d(Matern* c1, Matern* c2, void *state);
@@ -108,7 +110,9 @@ class Matern_Prior : public Corr_Prior
   virtual Base_Prior* BasePrior(void);
   virtual void SetBasePrior(Base_Prior *base_prior);
   virtual double log_HierPrior(void);
-  
+  virtual double* Trace(unsigned int* len);  
+  virtual char** TraceNames(unsigned int* len);
+  virtual void Init(double *dhier);
  
   double NU(void);
   double D(void);
