@@ -31,6 +31,7 @@ extern "C"
 #include "tgp.h"
 #include "model.h"
 #include "params.h"
+#include "mstructs.h"
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
@@ -44,12 +45,19 @@ Tgp* tgpm = NULL;
 void *tgp_state = NULL;
 
 void tgp(int* state_in, 
+
+	 /* inputs from R */
 	 double *X_in, int *n_in, int *d_in, double *Z_in, double *XX_in, int *nn_in,
 	 int *trace_in, int *BTE_in, int* R_in, int* linburn_in, double *params_in, 
-	 int *verb_in, double *Zp_mean_out, double *ZZ_mean_out, double *Zp_q_out, 
-	 double *ZZ_q_out, double *Zp_q1_out, double *Zp_median_out, double *Zp_q2_out,
-	 double *ZZ_q1_out, double *ZZ_median_out, double *ZZ_q2_out,
-	 double *Ds2x_out, double *ego_out)
+	 double *ditemps_in, int *verb_in, double *dtree_in, double* hier_in, 
+	 int *krige_in,
+
+	 /* outputs to R */
+	 double *Zp_mean_out, double *ZZ_mean_out, double *Zp_km_out, double *ZZ_km_out,
+	 double *Zp_q_out, double *ZZ_q_out, double *Zp_s2_out, double *ZZ_s2_out, 
+	 double *Zp_ks2_out, double *ZZ_ks2_out, double *Zp_q1_out, double *Zp_median_out, 
+	 double *Zp_q2_out, double *ZZ_q1_out, double *ZZ_median_out, double *ZZ_q2_out, 
+	 double *Ds2x_out, double *improv_out, double *ess_out)
 {
 
   /* create the RNG state */
@@ -58,18 +66,23 @@ void tgp(int* state_in,
 
   /* copy the input parameters to the tgp class object where all the MCMC 
      work gets done */
-  tgpm = new Tgp(tgp_state, *n_in, *d_in, *nn_in,
-		 BTE_in[0], BTE_in[1], BTE_in[2], *R_in, 
+  tgpm = new Tgp(tgp_state, *n_in, *d_in, *nn_in, BTE_in[0], BTE_in[1], BTE_in[2], *R_in, 
 		 *linburn_in, (bool) (Zp_mean_out!=NULL), (bool) (Ds2x_out!=NULL), 
-		 (bool) (ego_out != NULL), X_in, Z_in, XX_in, params_in, 
-		 (bool) *trace_in, *verb_in);
+		 (bool) (improv_out != NULL), X_in, Z_in, XX_in, params_in, ditemps_in, 
+		 (bool) *trace_in, *verb_in, dtree_in, hier_in);
 
   /* tgp MCMC rounds are done here */
-  tgpm->Rounds();
+  if(*krige_in) tgpm->Krige();
+  else tgpm->Rounds();
 
   /* gather the posterior predictive statistics from the MCMC rounds */
-  tgpm->GetStats(Zp_mean_out, ZZ_mean_out, Zp_q_out, ZZ_q_out, Zp_q1_out, Zp_median_out, 
-		 Zp_q2_out, ZZ_q1_out, ZZ_median_out, ZZ_q2_out, Ds2x_out, ego_out);
+  tgpm->GetStats(!((bool)*krige_in), Zp_mean_out, ZZ_mean_out, Zp_km_out, ZZ_km_out, 
+		 Zp_q_out, ZZ_q_out, Zp_s2_out, ZZ_s2_out, Zp_ks2_out, ZZ_ks2_out, 
+		 Zp_q1_out, Zp_median_out, Zp_q2_out, ZZ_q1_out, ZZ_median_out, 
+		 ZZ_q2_out, Ds2x_out, improv_out, ess_out);
+
+  /* write the modified tprobs back into the itemps vector */
+  dupv(&ditemps_in[1+(int)ditemps_in[0]], tgpm->get_iTemps()->tprobs, (int)ditemps_in[0]);
 
   /* delete the tgp model */
   delete tgpm; tgpm = NULL;
@@ -77,9 +90,6 @@ void tgp(int* state_in,
   /* destroy the RNG */
   deleteRNGstate(tgp_state);
   tgp_state = NULL;
-
-  /* free blank line before returning to R prompt */
-  // if(*verb_in >= 1) myprintf(stdout, "\n");
 }
 
 
@@ -91,9 +101,10 @@ void tgp(int* state_in,
  * function in order to get everything ready for MCMC rounds.
  */
 
-  Tgp::Tgp(void *state, int n, int d, int nn, int B, int T, int E, 
-	   int R, int linburn, bool pred_n, bool delta_s2, bool ego, double *X, 
-	   double *Z, double *XX, double *dparams, bool trace, int verb)
+Tgp::Tgp(void *state, int n, int d, int nn, int B, int T, int E, 
+	 int R, int linburn, bool pred_n, bool delta_s2, bool improv, double *X, 
+	 double *Z, double *XX, double *dparams, double *ditemps, bool trace, 
+	 int verb, double *dtree, double *hier)
 {
   itime = time(NULL);
 	
@@ -119,7 +130,9 @@ void tgp(int* state_in,
   this->linburn = linburn;
   this->pred_n = pred_n;
   this->delta_s2 = delta_s2;
-  this->ego = ego;
+  this->improv = improv;
+  assert(ditemps[0] >= 0);
+  this->itemps = new_itemps_double(ditemps);
   this->trace = trace;
   this->verb = verb;
 
@@ -139,7 +152,8 @@ void tgp(int* state_in,
   if((int) dparams[0] != -1) params->read_double(dparams);
   else myprintf(stdout, "Using default params.\n");
 
-  Init();
+  if(!dtree) Init(NULL, 0, hier);
+  else Init(&(dtree[1]), (unsigned int) dtree[0], hier);
 }
 
 
@@ -162,6 +176,7 @@ Tgp::~Tgp(void)
   if(X) { delete_matrix(X); X = NULL; }
   if(cumpreds) { delete_preds(cumpreds); }
   if(preds) { delete_preds(preds); }
+  if(itemps) { delete_itemps(itemps); }
 }
 
 
@@ -174,18 +189,18 @@ Tgp::~Tgp(void)
  * and predictive data.
  */
 
-void Tgp::Init(void)
+void Tgp::Init(double *tree, unsigned int ncol, double *hier)
 {
   /* get  the rectangle */
   rect = getXdataRect(X, n, d, XX, nn);
 
   /* construct the new model */
   model = new Model(params, d, rect, 0, trace, state);
-  model->Init(X, n, d, Z);
+  model->Init(X, n, d, Z, itemps, tree, ncol, hier);
   model->Outfile(stdout, verb);
   
   /* structure for accumulating predictive information */
-  cumpreds = new_preds(XX, nn, pred_n*n, d, rect, R*(T-B), delta_s2, ego, E);
+  cumpreds = new_preds(XX, nn, pred_n*n, d, rect, R*(T-B), delta_s2, improv, E);
   if(params->BasePrior()->BaseModel() == MR_GP)
     { for(unsigned int i=0; i<nn; i++) cumpreds->XX[i][0] = XX[i][0]; }
 
@@ -215,7 +230,7 @@ void Tgp::Rounds(void)
     model->Burnin(B, state);
 	
     /* do the MCMC rounds B,...,T */
-    preds = new_preds(XX, nn, pred_n*n, d, rect, T-B, delta_s2, ego, E);
+    preds = new_preds(XX, nn, pred_n*n, d, rect, T-B, delta_s2, improv, E);
     model->Sample(preds, T-B, state);
 
     /* print tree statistics */
@@ -225,10 +240,12 @@ void Tgp::Rounds(void)
     import_preds(cumpreds, preds->R * i, preds);		
     delete_preds(preds); preds = NULL;
 
-    /* done with this repetition; prune the tree all the way back */
+    /* done with this repetition; prune the tree all the way back 
+       and reset the inverse-temperatre probabilities */
     if(R > 1) {
-      myprintf(stdout, "finished repetition %d of %d\n", i+1, R);
+      if(verb >= 1) myprintf(stdout, "finished repetition %d of %d\n", i+1, R);
       model->cut_root();
+      dupv(itemps->tprobs, model->update_tprobs(), itemps->n);
     }
   }
 
@@ -242,11 +259,86 @@ void Tgp::Rounds(void)
   model->PrintPosteriors();
 
   /* this should only happen if trace==TRUE */
-  model->print_linarea();
+  model->PrintLinarea();
 
-  /* write the ZZ predictive data out to a file */
-  if(trace)
-    matrix_to_file("trace_ZZ_1.out", cumpreds->ZZ, cumpreds->R, nn);
+  /* write the preds out to files */
+  if(trace) {
+    if(nn > 0) {
+      matrix_to_file("trace_ZZ_1.out", cumpreds->ZZ, cumpreds->R, nn);
+      matrix_to_file("trace_ZZkm_1.out", cumpreds->ZZm, cumpreds->R, nn);
+      matrix_to_file("trace_ZZks2_1.out", cumpreds->ZZs2, cumpreds->R, nn);
+    }
+    if(pred_n) {
+      matrix_to_file("trace_Zp_1.out", cumpreds->Zp, cumpreds->R, n);
+      matrix_to_file("trace_Zpkm_1.out", cumpreds->Zpm, cumpreds->R, n);
+      matrix_to_file("trace_Zpks2_1.out", cumpreds->Zps2, cumpreds->R, n);
+    }
+    if(improv) matrix_to_file("trace_improv_1.out", cumpreds->improv, cumpreds->R, nn);
+
+    /* Ds2x is un-normalized, it needs to be divited by nn everywhere */
+    if(delta_s2) matrix_to_file("trace_Ds2x_1.out", cumpreds->Ds2x, cumpreds->R, nn);
+  }
+}
+
+
+/*
+ * Krige: 
+ *
+ * Only do sampling from the posterior predictive distribution;
+ * that is, don't update GP or Tree
+ */
+
+void Tgp::Krige(void)
+{
+  if(R > 1) warning("R=%d (>0) not necessary for Kriging", R);
+
+  for(unsigned int i=0; i<R; i++) {
+
+    itime = my_r_process_events(itime);
+
+    /* do the MCMC rounds B,...,T */
+    preds = new_preds(XX, nn, pred_n*n, d, rect, T-B, delta_s2, improv, E);
+    model->Krige(preds, T-B, state);
+
+    /* accumulate predictive information */
+    import_preds(cumpreds, preds->R * i, preds);		
+    delete_preds(preds); preds = NULL;
+
+    /* done with this repetition; prune the tree all the way back */
+    if(R > 1) {
+      myprintf(stdout, "finished repetition %d of %d\n", i+1, R);
+      // model->cut_root();
+    }
+  }
+
+  /* cap of the printing */
+  if(verb >= 1) myflush(stdout);
+
+  /* these is here to maintain compatibility with tgp::Rounds() */
+
+  /* print the rectangle of the MAP partition */
+  model->PrintBestPartitions();   
+
+  /* print the splits of the best tree for each height */
+  model->PrintPosteriors();
+
+  /* this should only happen if trace==TRUE */
+  model->PrintLinarea();
+
+  /* write the preds out to files */
+  if(trace) {
+    if(nn > 0) {
+      matrix_to_file("trace_ZZ_1.out", cumpreds->ZZ, cumpreds->R, nn);
+      matrix_to_file("trace_ZZkm_1.out", cumpreds->ZZm, cumpreds->R, nn);
+      matrix_to_file("trace_ZZks2_1.out", cumpreds->ZZs2, cumpreds->R, nn);
+    }
+    if(pred_n) {
+      matrix_to_file("trace_Zp_1.out", cumpreds->Zp, cumpreds->R, n);
+      matrix_to_file("trace_Zpkm_1.out", cumpreds->Zpm, cumpreds->R, n);
+      matrix_to_file("trace_Zpks2_1.out", cumpreds->Zps2, cumpreds->R, n);
+    }
+    if(improv) matrix_to_file("trace_improv_1.out", cumpreds->improv, cumpreds->R, nn);
+  }
 }
 
 
@@ -254,44 +346,94 @@ void Tgp::Rounds(void)
  * GetStats:
  *
  * Coalate the statistics from the samples of the posterior predictive
- * distribution gathered during the MCMC Tgp::Rounds() function.
+ * distribution gathered during the MCMC Tgp::Rounds() function
+ *
+ * argument indicates whether to report traces (e.g., for wess); i.e.,
+ * if Kriging (rather than Rounds) then parameters are fixed, so there 
+ * is no need for traces of weights because they should be constant
  */
 
-void Tgp::GetStats(double *Zp_mean, double *ZZ_mean, double *Zp_q, double *ZZ_q,
-	      double *Zp_q1, double *Zp_median, double *Zp_q2, double *ZZ_q1, 
-	      double *ZZ_median, double *ZZ_q2, double *Ds2x, double *ego)
+void Tgp::GetStats(bool report, double *Zp_mean, double *ZZ_mean, 
+		   double *Zp_km, double *ZZ_km, double *Zp_q, double *ZZ_q, 
+		   double *Zp_s2, double *ZZ_s2, double *Zp_ks2, double *ZZ_ks2, 
+		   double *Zp_q1, double *Zp_median, double *Zp_q2, double *ZZ_q1, 
+		   double *ZZ_median, double *ZZ_q2, double *Ds2x, double *improv,
+		   double *ess)
 {
   itime = my_r_process_events(itime);
 
-  /* calculate means and quantiles */
-  if(pred_n) {
-    mean_of_columns(Zp_mean, cumpreds->Zp, cumpreds->R, n);
-    qsummary(Zp_q, Zp_q1, Zp_median, Zp_q2, cumpreds->Zp, cumpreds->R, n);
+  /* adjust weights by within-temperature ESS */
+  double *w = NULL;
+  if(itemps->n > 1 || itemps->itemps[0] != 1.0) {
+    *ess = lambda_ess(itemps, cumpreds->w, cumpreds->itemp, cumpreds->R);
+    if(trace && report) vector_to_file("trace_wess_1.out", cumpreds->w, cumpreds->R);
+    w = cumpreds->w;
+  } else {
+    *ess = cumpreds->R;
   }
 
-  /* means and wuantiles at predictive data locations (XX) */
-  if(nn > 0) { 
-    mean_of_columns(ZZ_mean, cumpreds->ZZ, cumpreds->R, nn);
-    qsummary(ZZ_q, ZZ_q1, ZZ_median, ZZ_q2, cumpreds->ZZ, cumpreds->R, cumpreds->nn);
+  /* allcoate pointers for holding q1 median and q3 */
+  double q[3] = {0.05, 0.5, 0.95};
+  double **Q = (double**) malloc(sizeof(double*) * 3);
+
+  /* calculate means and quantiles */
+  if(pred_n) {
+    assert(n == cumpreds->n);
     
-    /* expected retuduction in squared error */
-    /* warning: this makes a permanent change to cumpreds->Ds2xy, 
-                should change this! */
-    if(cumpreds->Ds2xy) norm_Ds2xy(cumpreds->Ds2xy, cumpreds->R, cumpreds->nn);
-   
-    /* expected reduction in squared error,
-       averaged over the Y locations */
-    if(delta_s2) {
-      assert(cumpreds->Ds2xy);
-      mean_of_rows(Ds2x, cumpreds->Ds2xy, nn, nn);
+    /* mean */
+    wmean_of_columns(Zp_mean, cumpreds->Zp, cumpreds->R, n, w);
+
+    /* kriging mean */
+    wmean_of_columns(Zp_km, cumpreds->Zpm, cumpreds->R, n, w);
+
+    /* variance (computed from samples Zp) */
+    wmean_of_columns_f(Zp_s2, cumpreds->Zp, cumpreds->R, n, w, sq);
+    for(unsigned int i=0; i<n; i++) Zp_s2[i] -= sq(Zp_mean[i]);
+
+    /* kriging variance */
+    wmean_of_columns(Zp_ks2, cumpreds->Zps2, cumpreds->R, n, w);
+
+    /* quantiles and medians */
+    Q[0] = Zp_q1; Q[1] = Zp_median; Q[2] = Zp_q2;
+    quantiles_of_columns(Q, q, 3, cumpreds->Zp, cumpreds->R, n, w);
+    for(unsigned int i=0; i<n; i++) Zp_q[i] = Zp_q2[i]-Zp_q1[i];
+  }
+
+  /* means and quantiles at predictive data locations (XX) */
+  if(nn > 0) {
+    
+    /* mean */
+    wmean_of_columns(ZZ_mean, cumpreds->ZZ, cumpreds->R, nn, w);
+
+    /* kriging mean */
+    wmean_of_columns(ZZ_km, cumpreds->ZZm, cumpreds->R, nn, w);
+
+    /* variance (computed from samples ZZ) */
+    wmean_of_columns_f(ZZ_s2, cumpreds->ZZ, cumpreds->R, nn, w, sq);
+    for(unsigned int i=0; i<nn; i++) ZZ_s2[i] -= sq(ZZ_mean[i]);
+
+    /* kriging variance */
+    wmean_of_columns(ZZ_ks2, cumpreds->ZZs2, cumpreds->R, nn, w);
+
+    /* quantiles and medians */
+    Q[0] = ZZ_q1; Q[1] = ZZ_median; Q[2] = ZZ_q2;
+    quantiles_of_columns(Q, q, 3, cumpreds->ZZ, cumpreds->R, cumpreds->nn, w);
+    for(unsigned int i=0; i<nn; i++) ZZ_q[i] = ZZ_q2[i]-ZZ_q1[i];
+    
+    if(cumpreds->Ds2x) {
+      assert(delta_s2);
+      wmean_of_columns(Ds2x, cumpreds->Ds2x, cumpreds->R, cumpreds->nn, w);
     }
 
     /* expected global optimum (minima) */
-    if(ego) {
-      scalev(cumpreds->ego, cumpreds->nn, 1.0/cumpreds->R);
-      dupv(ego, cumpreds->ego, nn);
+    if(improv) {
+      assert(cumpreds->improv);
+      wmean_of_columns(improv, cumpreds->improv, cumpreds->R, cumpreds->nn, w);
     }
   }
+
+  /* clean up */
+  free(Q);
 }
 
 
@@ -367,11 +509,11 @@ void Tgp::Print(FILE *outfile)
   printRNGstate(state, stdout);
 
   /* print predictive statistic types */
-  if(pred_n || delta_s2 || ego) myprintf(stdout, "preds:");
+  if(pred_n || delta_s2 || improv) myprintf(stdout, "preds:");
   if(pred_n) myprintf(stdout, " data");
   if(delta_s2) myprintf(stdout, " ALC");
-  if(ego) myprintf(stdout, " EGO");
-  if(pred_n || delta_s2 || ego) myprintf(stdout, "\n");
+  if(improv) myprintf(stdout, " IMPROV");
+  if(pred_n || delta_s2 || improv) myprintf(stdout, "\n");
   myflush(stdout);
 
   /* print the model, uses the internal model 
@@ -390,3 +532,15 @@ int Tgp::Verb(void)
 {
   return verb;
 }
+
+
+/*
+ * iTemps:
+ *
+ * return a pointer to the itemps structure
+ */
+
+iTemps* Tgp::get_iTemps(void)
+{
+  return itemps;
+}
