@@ -46,12 +46,13 @@ class Tgp
   unsigned int T;        /* total number of MCMC rounds (including burn-in) */
   unsigned int E;        /* sample from posterior (E)very somany rounds */
   unsigned int R;        /* number of times to (Re-) start over (>=1) */
+  iTemps *itemps;        /* set of inv-temperatures for importance tempering */
   int verb;              /* indicates the verbosity of print statements */
 
   bool linburn;          /* initialize with treed LM before burn in? */
   bool pred_n;           /* sample from posterior predictive at data locs? */
   bool delta_s2;         /* gather ALC statistics? */
-  bool ego;              /* gather EGO statistics? */
+  bool improv;           /* gather IMPROV statistics? */
   
   double **X;            /* n-by-d input (design matrix) data */
   double *Z;             /* response vector of length n */
@@ -61,25 +62,32 @@ class Tgp
   double **rect;         /* bounding rectangle of the (design matrix) data X */
   Model *model;          /* pointer to the (treed GP) model */
   Preds *cumpreds;       /* data structure for gathering posterior pred samples */
-  Preds *preds;          /* temporary for posteior pred samples */
-  
-  void Init(void);       /* function that should only be called from constructor */
+  Preds *preds;          /* inv-temporary for posteior pred samples */
+
+  /* a function that should only be called from constructor */  
+  void Init(double *tree, unsigned int ncol, double *hier);       
 
  public:
 
   Tgp(void *state, int n, int d, int nn, int B, int T, int E, int R, 
-      int linburn, bool pred_n, bool delta_s2, bool ego, double *X, double *Z, 
-      double *XX, double *dparams, bool trace, int verb);
+      int linburn, bool pred_n, bool delta_s2, bool improv, double *X, double *Z, 
+      double *XX, double *dparams, double *ditemps, bool trace, int verb, 
+      double *dtree, double *hier);
   ~Tgp(void);
   void Rounds(void);
-  void GetStats(double *Zp_mean, double *ZZ_mean, double *Zp_q, double *ZZ_q,
+  void Krige(void);
+  void GetStats(bool report, double *Zp_mean, double *ZZ_mean, 
+		double *Zp_km, double *ZZ_km, double *Zp_q, double *ZZ_q, 
+		double *Zp_s2, double *ZZ_s2, double *Zp_ks2, double *ZZ_ks2, 
 		double *Zp_q1, double *Zp_median, double *Zp_q2, double *ZZ_q1, 
-		double *ZZ_median, double *ZZ_q2, double *Ds2x, double *ego);
+		double *ZZ_median, double *ZZ_q2, double *Ds2x, double *improv,
+		double *ess);
   void Print(FILE *outfile);
   int Verb(void);
+  iTemps* get_iTemps(void);
 };
 
-
+void norm_Ds2xy(double **Ds2xy, double R, unsigned int n);
 double ** getXdataRect(double **X, unsigned int n, unsigned int d, double **XX, 
 		       unsigned int nn);
 
